from scraipe.classes import IScraper, ScrapeResult
import requests
from bs4 import BeautifulSoup

class DefaultScraper(IScraper):
    """The default scraper that pulls the content of a webpage using requests and filters out html tags."""
    DEFAULT_USER_AGENT = "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/58.0.3029.110 Safari/537.36"
    def __init__(self, headers=None):
        self.headers = headers or {"User-Agent": DefaultScraper.DEFAULT_USER_AGENT}
    
    def scrape(self, url:str)->ScrapeResult:
        try:
            response = requests.get(url, headers=self.headers)
            if response.status_code != 200:
                return ScrapeResult(
                    link=url,
                    content=None,
                    scrape_success=False, 
                    scrape_error=f"Failed to scrape {url}. Status code: {response.status_code}")
            text = response.text
            # Use bs4 to extract the text from the html
            soup = BeautifulSoup(text, "html.parser")
            content = soup.get_text()
            # Remove multiple consecutive lines
            content = "\n".join([line for line in content.split("\n") if line.strip() != ""])
            
            return ScrapeResult(link=url, content=content, scrape_success=True)
        except Exception as e:
            return ScrapeResult(link=url,scrape_success=False, scrape_error=f"Failed to scrape {url}. Error: {e}")


from scraipe.classes import IScraper, ScrapeResult
import requests
from bs4 import BeautifulSoup
from newspaper import Article
from telethon import TelegramClient
import re
import asyncio

class TelegramScraper(IScraper):
    """A scraper that uses the newspaper3k library to extract article content."""
    client: TelegramClient
    DEFAULT_USER_AGENT = "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/58.0.3029.110 Safari/537.36"
    def __init__(self, name: str, api_id: str, api_hash: str, phone_number: str):
        self.client = TelegramClient(name, api_id, api_hash)  # Assign to self.client
        self.client.start(phone_number)
        
    async def _get_telegram_content(self, chat_name:str, message_id:int):
        await self.client.connect()
        # Search for entity
        try:
            entity = await self.client.get_entity(chat_name)
        except Exception as e:
            raise Exception(f"Failed to get entity for {chat_name}: {e}")
        
        # Check if entity is restricted/unaccessible
        if entity.restricted:
            raise Exception(f"Entity {chat_name} is restricted.")
        
        # Get the message from entity
        try:
            message = await self.client.get_messages(entity, ids=message_id)  # Fix variable name
        except Exception as e:
            raise Exception(f"Failed to get message {message_id} from {chat_name}: {e}")
        return message.message

    def scrape(self, url: str) -> ScrapeResult:  # Make scrape an async method
        if not url.startswith("https://t.me/"):
            return ScrapeResult(link=url, scrape_success=False, scrape_error=f"URL {url} is not a telegram link.")
        # Extract the username and message id
        match = re.match(r"https://t.me/([^/]+)/(\d+)", url)
        if not match:
            error = f"Failed to extract username and message id from {url}"
            return ScrapeResult(link=url, scrape_success=False, scrape_error=error)
        username, message_id = match.groups()
        try:
            message_id = int(message_id)
        except ValueError:
            raise ValueError(f"Failed to convert message id {message_id} to int.")
        # Run async function
        try:
            content = asyncio.get_event_loop().run_until_complete(self._get_telegram_content(username, message_id))
        except Exception as e:
            return ScrapeResult(link=url, scrape_success=False, scrape_error=f"Failed to scrape {url}. Error: {e}")
        
        return ScrapeResult(link=url, content=content, scrape_success=True)
    
    def disconnect(self):
        self.client.disconnect()
    



import pytest
from unittest.mock import patch, MagicMock
from scraipe.scrapers.default_scraper import DefaultScraper
from scraipe import ScrapeResult,AnalysisResult


def test_default_headers():
    scraper = DefaultScraper()
    assert scraper.headers == {"User-Agent": DefaultScraper.DEFAULT_USER_AGENT}


def test_custom_headers():
    custom_headers = {"User-Agent": "CustomAgent/1.0"}
    scraper = DefaultScraper(headers=custom_headers)
    assert scraper.headers == custom_headers


@patch("scraipe.scrapers.default_scraper.requests.get")
def test_scrape_success(mock_get):
    mock_response = MagicMock()
    mock_response.text = "Mocked response content"
    mock_response.status_code = 200
    mock_get.return_value = mock_response

    scraper = DefaultScraper()
    scrape_result:ScrapeResult = scraper.scrape("https://google.com")
    assert scrape_result.scrape_success, f"Scrape failed: {scrape_result.scrape_error}"
    assert scrape_result.content == "Mocked response content"
    mock_get.assert_called_once_with("https://google.com", headers=scraper.headers)


@patch("scraipe.scrapers.default_scraper.requests.get")
def test_scrape_failure(mock_get):
    mock_get.side_effect = Exception("Request failed")

    scraper = DefaultScraper()
    result = scraper.scrape("https://invalid-url-aoietasdnlkzbxjcnweaituh.com")
    assert result.scrape_success == False
    
def test_scrape_google():
    # Check if connection to google succeeds
    import requests
    if not requests.get("https://www.google.com").status_code == 200:
        pytest.skip("Could not connect to google.com")
        return
        
    scraper = DefaultScraper()
    result = scraper.scrape("https://www.google.com")
    assert result.scrape_success
    assert isinstance(result.content, str)
    assert len(result.content) > 0


from scraipe.classes import IAnalyzer, AnalysisResult
from openai import OpenAI
import json
from pydantic import BaseModel, ValidationError
from typing import Type

class OpenAiAnalyzer(IAnalyzer):
    """An analyzer that uses the OpenAI API to analyze text and returns a json parsed dict"""
    
    # Attributes
    client:OpenAI
    """The OpenAI instance to use for querying the OpenAI API"""
    instruction:str
    """The instruction or prompt to guide the OpenAI model's behavior"""
    pydantic_schema:Type[BaseModel]
    """A Pydantic schema to validate the model's output"""
    model:str
    """The name of the OpenAI model to use"""
    
    def __init__(self,
        api_key:str,
        instruction:str,
        organization:str=None,
        pydantic_schema:Type[BaseModel] = None,
        model:str="gpt-4o-mini"):
        """Initializes the OpenAIAnalyzer instance."""
        self.api_key = api_key
        self.client = OpenAI(api_key=api_key,organization=organization)
        self.instruction = instruction
        self.pydantic_schema = pydantic_schema
        self.model = model
    
    def query_openai(self, content:str) -> str:
        """Queries the OpenAI API with the content and configured instruction."""
        # Send the content with the instruction as a system instruction
        messages=[
            {"role": "system", "content": self.instruction},
            {"role": "user", "content": content}
        ]
        response = self.client.chat.completions.create(
            model=self.model,
            messages=messages,
            response_format={ "type": "json_object" }
        )
        response_content:str = response.choices[0].message.content
        return response_content

    def analyze(self, content:str) -> AnalysisResult:
        """Analyzes the content using the OpenAI API and returns the response as a dict."""
        try:
            response = self.query_openai(content)
        except Exception as e:
            return AnalysisResult(output=None, analysis_success=False, analysis_error=f"Failed to query OpenAI: {e}")
        
        # Check if response is json string
        try:
            response_dict = json.loads(response)
        except json.JSONDecodeError:
            return AnalysisResult(analysis_success=False, analysis_error=f"OpenAI response is not a valid json string: {response}")
        
        # Check if response follows the pydantic schema
        if self.pydantic_schema:
            try:
                self.pydantic_schema(**response_dict)
            except ValidationError as e:
                return AnalysisResult(output=response_dict, analysis_success=False, analysis_error=f"OpenAI response does not follow the pydantic schema: {e}")
        
        return AnalysisResult(output=response_dict, analysis_success=True)


import os
import pytest
from scraipe.scrapers.multi_scraper import MultiScraper
from scraipe.scrapers.telegram_scraper import TelegramScraper
from scraipe.scrapers.news_scraper import NewsScraper
from scraipe.scrapers.default_scraper import DefaultScraper
from scraipe.classes import ScrapeResult
from unittest.mock import MagicMock


TEST_TELEGRAM_URL = "https://t.me/TelegramTips/516"

@pytest.fixture
def mock_telegram_scraper():
    # Attempt to create a live Telegram scraper
    name = os.environ.get("TELEGRAM_NAME")
    api_id = os.environ.get("TELEGRAM_API_ID")
    api_hash = os.environ.get("TELEGRAM_API_HASH")
    phone_number = os.environ.get("TELEGRAM_PHONE_NUMBER")

    if all([name, api_id, api_hash, phone_number]):
        try:
            scraper = TelegramScraper(name, api_id, api_hash, phone_number)
            yield scraper
            scraper.disconnect()
            return
        except Exception as e:
            print(f"Failed to create live Telegram scraper: {e}")

    # Fallback to mock scraper
    class MockTelegramScraper:
        def scrape(self, url):
            if "valid" in url:
                return ScrapeResult(link=url, scrape_success=True, content="Mocked Telegram content")
            return ScrapeResult(link=url, scrape_success=False, scrape_error="Invalid Telegram link")
    
    yield MockTelegramScraper()

@pytest.fixture
def mock_news_scraper():
    class MockNewsScraper:
        def scrape(self, url):
            if "news" in url:
                return ScrapeResult(link=url, scrape_success=True, content="News content")
            return ScrapeResult(link=url, scrape_success=False, scrape_error="Not a news link")
    return MockNewsScraper()

@pytest.fixture
def mock_default_scraper():
    class MockDefaultScraper:
        def scrape(self, url):
            return ScrapeResult(link=url, scrape_success=True, content="Default content")
    return MockDefaultScraper()

@pytest.fixture
def multi_scraper(mock_telegram_scraper, mock_news_scraper, mock_default_scraper):
    return MultiScraper(
        telegram_scraper=mock_telegram_scraper,
        news_scraper=mock_news_scraper,
        default_scraper=mock_default_scraper
    )

def test_telegram_scraper(multi_scraper):
    if isinstance(multi_scraper.telegram_scraper, TelegramScraper):
        url = TEST_TELEGRAM_URL
    else:
        url = "https://t.me/valid"
    result = multi_scraper.scrape(url)
    assert result.success
    assert len(result.content) > 0

def test_news_scraper(multi_scraper):
    url = "https://example.com/news"
    result = multi_scraper.scrape(url)
    assert result.success
    assert result.content == "News content"

def test_default_scraper(multi_scraper):
    url = "https://example.com/other"
    result = multi_scraper.scrape(url)
    assert result.success
    assert result.content == "Default content"